diff --git a/encoder/analyse.c b/encoder/analyse.c
index 70c6c19..af3185f 100644
--- a/encoder/analyse.c
+++ b/encoder/analyse.c
@@ -1543,8 +1543,11 @@ static void x264_mb_analyse_inter_b16x16( x264_t *h, x264_mb_analysis_t *a )
     a->l1.me16x16.cost += a->i_lambda * i_mb_b_cost_table[B_L1_L1];
 }
 
-static inline void x264_mb_cache_mv_p8x8( x264_t *h, x264_mb_analysis_t *a, int i )
+static inline void x264_mb_cache_mv_p8x8( x264_t *h, x264_mb_analysis_t *a, int i, int b_mvd )
 {
+    int mvp[2];
+    int mdx, mdy;
+
     const int x = 2*(i%2);
     const int y = 2*(i/2);
 
@@ -1556,16 +1559,62 @@ static inline void x264_mb_cache_mv_p8x8( x264_t *h, x264_mb_analysis_t *a, int
         case D_L0_8x4:
             x264_macroblock_cache_mv( h, x, y+0, 2, 1, 0, a->l0.me8x4[i][0].mv[0], a->l0.me8x4[i][0].mv[1] );
             x264_macroblock_cache_mv( h, x, y+1, 2, 1, 0, a->l0.me8x4[i][1].mv[0], a->l0.me8x4[i][1].mv[1] );
+            if( b_mvd )
+            {
+                x264_mb_predict_mv( h, 0, i*4+0, 2, mvp );
+                mdx = a->l0.me8x4[i][0].mv[0] - mvp[0];
+                mdy = a->l0.me8x4[i][0].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x, y+0, 2, 1, 0, mdx, mdy );
+
+                x264_mb_predict_mv( h, 0, i*4+2, 2, mvp );
+                mdx = a->l0.me8x4[i][1].mv[0] - mvp[0];
+                mdy = a->l0.me8x4[i][1].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x, y+1, 2, 1, 0, mdx, mdy );
+            }
             break;
         case D_L0_4x8:
             x264_macroblock_cache_mv( h, x+0, y, 1, 2, 0, a->l0.me4x8[i][0].mv[0], a->l0.me4x8[i][0].mv[1] );
             x264_macroblock_cache_mv( h, x+1, y, 1, 2, 0, a->l0.me4x8[i][1].mv[0], a->l0.me4x8[i][1].mv[1] );
+            if( b_mvd )
+            {
+                x264_mb_predict_mv( h, 0, i*4+0, 1, mvp );
+                mdx = a->l0.me4x8[i][0].mv[0] - mvp[0];
+                mdy = a->l0.me4x8[i][0].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x+0, y, 1, 2, 0, mdx, mdy );
+
+                x264_mb_predict_mv( h, 0, i*4+1, 1, mvp );
+                mdx = a->l0.me4x8[i][1].mv[0] - mvp[0];
+                mdy = a->l0.me4x8[i][1].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x+1, y, 1, 2, 0, mdx, mdy );
+            }
             break;
         case D_L0_4x4:
             x264_macroblock_cache_mv( h, x+0, y+0, 1, 1, 0, a->l0.me4x4[i][0].mv[0], a->l0.me4x4[i][0].mv[1] );
             x264_macroblock_cache_mv( h, x+1, y+0, 1, 1, 0, a->l0.me4x4[i][1].mv[0], a->l0.me4x4[i][1].mv[1] );
             x264_macroblock_cache_mv( h, x+0, y+1, 1, 1, 0, a->l0.me4x4[i][2].mv[0], a->l0.me4x4[i][2].mv[1] );
             x264_macroblock_cache_mv( h, x+1, y+1, 1, 1, 0, a->l0.me4x4[i][3].mv[0], a->l0.me4x4[i][3].mv[1] );
+            if( b_mvd )
+            {
+                x264_mb_predict_mv( h, 0, i*4+0, 1, mvp );
+                mdx = a->l0.me4x4[i][0].mv[0] - mvp[0];
+                mdy = a->l0.me4x4[i][0].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x+0, y+0, 1, 1, 0, mdx, mdy );
+
+                x264_mb_predict_mv( h, 0, i*4+1, 1, mvp );
+                mdx = a->l0.me4x4[i][1].mv[0] - mvp[0];
+                mdy = a->l0.me4x4[i][1].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x+1, y+0, 1, 1, 0, mdx, mdy );
+
+                x264_mb_predict_mv( h, 0, i*4+2, 1, mvp );
+                mdx = a->l0.me4x4[i][2].mv[0] - mvp[0];
+                mdy = a->l0.me4x4[i][2].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x+0, y+1, 1, 1, 0, mdx, mdy );
+
+                x264_mb_predict_mv( h, 0, i*4+3, 1, mvp );
+                mdx = a->l0.me4x4[i][3].mv[0] - mvp[0];
+                mdy = a->l0.me4x4[i][3].mv[1] - mvp[1];
+                x264_macroblock_cache_mvd( h, x+1, y+1, 1, 1, 0, mdx, mdy );
+            }
             break;
         default:
             x264_log( h, X264_LOG_ERROR, "internal error\n" );
@@ -2159,7 +2208,7 @@ void x264_macroblock_analyse( x264_t *h )
 
                             i_cost += i_cost8x8 - analysis.l0.me8x8[i].cost;
                         }
-                        x264_mb_cache_mv_p8x8( h, &analysis, i );
+                        x264_mb_cache_mv_p8x8( h, &analysis, i, 0 );
                     }
                     analysis.l0.i_cost8x8 = i_cost;
                 }
@@ -2326,6 +2375,8 @@ void x264_macroblock_analyse( x264_t *h )
                     for( i8x8 = 0; i8x8 < 4; i8x8++ )
                          if( h->mb.i_sub_partition[i8x8] == D_L0_8x8 )
                              x264_me_refine_qpel_rd( h, &analysis.l0.me8x8[i8x8], analysis.i_lambda2, i8x8 );
+                         else
+                             x264_mb_cache_mv_p8x8( h, &analysis, i8x8, 1 );
                 }
             }
         }
@@ -2639,7 +2690,7 @@ static void x264_analyse_update_cache( x264_t *h, x264_mb_analysis_t *a  )
             x264_macroblock_cache_ref( h, 0, 2, 2, 2, 0, a->l0.me8x8[2].i_ref );
             x264_macroblock_cache_ref( h, 2, 2, 2, 2, 0, a->l0.me8x8[3].i_ref );
             for( i = 0; i < 4; i++ )
-                x264_mb_cache_mv_p8x8( h, a, i );
+                x264_mb_cache_mv_p8x8( h, a, i, 0 );
             break;
 
         case P_SKIP: